﻿using System;
using System.Windows;
using System.Windows.Controls;
using Stun2DPhysics4SL;
using Stun2DPhysics4SL.Dynamics;
using Stun2DPhysics4SL.Mathematics;

namespace Stun2DPhysics4SLDemo
{
    public abstract class Sprite
    {
        // 与Sprite链接的UserControl对象
        protected UserControl ucSpriteUC;
        
        // 父Canvas控件
        protected Canvas cnvParent;
        
        // 获取或设置UseControl的位置
        public Point Location { get;set;}

        public Sprite(Canvas setCnvParent, Point initialLocation)
        {
            cnvParent = setCnvParent;
            ucSpriteUC = CreateSpriteUC();
            Location = initialLocation;

            cnvParent.Children.Add(ucSpriteUC);
            
            // 设置UseControl的初始位置
            ucSpriteUC.SetValue(Canvas.LeftProperty, Location.X);
            ucSpriteUC.SetValue(Canvas.TopProperty, Location.Y);
        }

        public abstract UserControl CreateSpriteUC();

        public virtual void Update()
        {            
        }
    }

    public abstract class SpritePhysice : Sprite
    {
        public Body Body { get; private set; }

        public SpritePhysice(Canvas cnvParent, Point initialLocation, PhysicsSimulator physicsSimulator)
            : base(cnvParent, initialLocation)
        {
            Body = new Body();
            Body.Position = new Vector2((float)initialLocation.X, (float)initialLocation.Y);
            physicsSimulator.Add(Body);
        }

        public override void Update()
        {
            ucSpriteUC.SetValue(Canvas.LeftProperty, Convert.ToDouble(Body.Position.X));
            ucSpriteUC.SetValue(Canvas.TopProperty, Convert.ToDouble(Body.Position.Y));
        }
    }

    public class Box1Sprite : SpritePhysice
    {
        public Box1Sprite(Canvas cnvParent, Point initialLocation, PhysicsSimulator physicsSimulator)
            : base(cnvParent, initialLocation,physicsSimulator)
        {
            
        }

        public override UserControl CreateSpriteUC()
        {
            return new Box1();
        }
    }

    public class Box2Sprite : SpritePhysice
    {
        public Box2Sprite(Canvas cnvParent, Point initialLocation, PhysicsSimulator physicsSimulator)
            : base(cnvParent, initialLocation, physicsSimulator)
        {

        }

        public override UserControl CreateSpriteUC()
        {
            return new Box2();
        }
    }
}
